@@ -93,7 +93,9 @@ func TestNodeFromFileInfo(t *testing.T) {
 
 	// on darwin, users are not permitted to list the extended attributes of
 	// /dev/null, therefore skip it.
-	if runtime.GOOS != "darwin" {
+	// on solaris, /dev/null is a symlink to a device node in /devices
+	// which does not support extended attributes, therefore skip it.
+	if runtime.GOOS != "darwin" && runtime.GOOS != "solaris" {
 		tests = append(tests, Test{"/dev/null", true})
 	}
 
